package com.yyge.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 针对扣减库存的数据不一致的BUG进行的优化
 * 新增了synchronized锁住代码块
 * 但是依旧存在问题：synchronized是在是单机状态下并发问题解决方案
 * 但是在分布式集群的环境下synchronized就会出现问题
 * 解决方案：使用分布式锁
 */
@RestController
public class resolve1 {

    @Autowired
    StringRedisTemplate stringRedisTemplate;

    @RequestMapping("/deduct_stock")
    public String deductStock(){
        //加锁
        synchronized (this){
            //查询redis里的库存 假设查询结果是200
            int stock = Integer.parseInt(stringRedisTemplate.opsForValue().get("stock"));

            //如果库存结果大于0
            if(stock > 0){
                int realStock = stock - 1; //库存-1  结果为199
                stringRedisTemplate.opsForValue().set("stock", String.valueOf(realStock)); //重新设置库存
                System.out.println("扣减成功，剩余库存:" + realStock); //输出199
            }else{
                //否则库存扣减失败
                System.out.println("扣减失败，库存不足");
            }
            return "end";
        }
    }
}
