package com.yyge.controller;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

/**
 * 进一步对代码进行优化-使用redis的SETNX 命令来实现分布式锁
 * stringRedisTemplate.opsForValue().setIfAbsent(lockKey, "yyge");
 *
 * 但是即使如此依旧存在问题
 * 在高并发情况下,有一种场景,分布式锁添加成功
 * 但是在处理业务逻辑中发生了异常，导致未能正常执行释放锁的操作
 * 此时这个锁就会一直存在....
 *
 * 解决方案：捕获异常
 */

@RestController
public class resolve2 {
    @Autowired
    StringRedisTemplate stringRedisTemplate;

    @RequestMapping("/deduct_stock")
    public String deductStock(){
        String lockKey="lock:product:id"; //根据商品ID生产一个KEY,商品ID是前端传过来的
        //添加分布式锁进行加锁  -- 如果lockKey 不存在 则设置值返回true，如果存在则不会做任何操作返回false
        //添加分布式锁setNX
        Boolean result = stringRedisTemplate.opsForValue().setIfAbsent(lockKey, "yyge");
        if (Boolean.FALSE.equals(result)){
            //如果加锁失败则返回前端提示
            return "biz_code";
        }

        //查询redis里的库存 假设查询结果是200
        int stock = Integer.parseInt(stringRedisTemplate.opsForValue().get("stock"));

        //如果库存结果大于0
        if(stock > 0){
            int realStock = stock - 1; //库存-1  当在高并发情况下多个线程都扣减1 200-1结果为199
            stringRedisTemplate.opsForValue().set("stock", String.valueOf(realStock)); //重新设置库存
            System.out.println("扣减成功，剩余库存:" + realStock); //线程1、线程2...都输出199
            //出现数据不一致的BUG问题
        }else{
            //否则库存扣减失败
            System.out.println("扣减失败，库存不足");
        }

        //删除分布式锁
        stringRedisTemplate.delete(lockKey);
        return "end";
    }

}
