package com.yyge.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.TimeoutUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import java.util.concurrent.TimeUnit;
/**
 * 进一步优化：为了解决在执行业务逻辑代码时服务器宕机导致分布式锁未释放的问题
 * 我们在加锁的时候设置一个过期时间来防止这个问题
 *
 * 如此操作还是有问题：
 * 我们的业务流程收到各个方面的影响,执行时间是无法预料的
 * 如果业务代码的执行时间超过了锁的过期时间,那么就会出现其他线程释放锁的问题
 * 也就是说 上锁和释放锁的不是一个线程
 *
 * 解决方案：将线程ID或者UUID设置为锁的值
 */

@RestController
public class resolve3 {

    @Autowired
    StringRedisTemplate stringRedisTemplate;

    @RequestMapping("/deduct_stock")
    public String deductStock(){
        String lockKey="lock:product:id"; //根据商品ID生产一个KEY,商品ID是前端传过来的
        //添加分布式锁进行加锁  -- 如果lockKey 不存在 则设置值返回true，如果存在则不会做任何操作返回false
        //添加分布式锁setNX,并设置过期时间10秒钟
        Boolean result = stringRedisTemplate.opsForValue().setIfAbsent(lockKey, "yyge",10,TimeUnit.SECONDS);
        if (Boolean.FALSE.equals(result)){
            //如果加锁失败则返回前端提示
            return "biz_code";
        }
        try {
            //查询redis里的库存 假设查询结果是200
            int stock = Integer.parseInt(stringRedisTemplate.opsForValue().get("stock"));

            //如果库存结果大于0
            if(stock > 0){
                int realStock = stock - 1; //库存-1  结果为199
                stringRedisTemplate.opsForValue().set("stock", String.valueOf(realStock)); //重新设置库存
                System.out.println("扣减成功，剩余库存:" + realStock); //输出199
            }else{
                //否则库存扣减失败
                System.out.println("扣减失败，库存不足");
            }
        }finally {
            //删除分布式锁
            stringRedisTemplate.delete(lockKey);
        }
        return "end";
    }

}
