package com.yyge.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 我们进一步优化：为了解决在分布式情况下,当前的锁被别的线程释放的问题
 * 我们在设置锁的时候存入线程ID作为值
 *
 * 依然存在问题：
 * 当我们判断线程ID的时候如果服务器宕机，会导致锁一直不会被释放
 *
 * 解决方案：使用Lua保证原子性...
 */

@RestController
public class resolve4 {

    @Autowired
    StringRedisTemplate stringRedisTemplate;

    @RequestMapping("/deduct_stock")
    public String deductStock(){
        String lockKey="lock:product:id"; //根据商品ID生产一个KEY,商品ID是前端传过来的
        //获取当前线程ID ！！！这里也可以换成UUID
        String theadId = String.valueOf(Thread.currentThread().getId());

        //添加分布式锁进行加锁  -- 如果lockKey 不存在 则设置值返回true，如果存在则不会做任何操作返回false
        //添加分布式锁setNX,并设置过期时间10秒钟
        Boolean result = stringRedisTemplate.opsForValue().setIfAbsent(lockKey, theadId,10,TimeUnit.SECONDS);
        if (Boolean.FALSE.equals(result)){
            //如果加锁失败则返回前端提示
            return "biz_code";
        }
        try {
            //查询redis里的库存 假设查询结果是200
            int stock = Integer.parseInt(stringRedisTemplate.opsForValue().get("stock"));

            //如果库存结果大于0
            if(stock > 0){
                int realStock = stock - 1; //库存-1  结果为199
                stringRedisTemplate.opsForValue().set("stock", String.valueOf(realStock)); //重新设置库存
                System.out.println("扣减成功，剩余库存:" + realStock); //输出199
            }else{
                //否则库存扣减失败
                System.out.println("扣减失败，库存不足");
            }
        }finally {
            //判断当前的线程ID是否是当前Key的value值,如果是则删除锁
            if(theadId.equals(stringRedisTemplate.opsForValue().get(lockKey))){
                //删除分布式锁
                stringRedisTemplate.delete(lockKey);
            }

        }
        return "end";
    }
}
